varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;

uniform int direction;
uniform int colorMode;
uniform lowp float offset;
uniform lowp float vDisplacement;
uniform lowp float phaseShift;// 0.0 ~ 1.0
uniform lowp float amplitude;// 0.0 ~ 1.0
uniform lowp float noiseScale;// 0.0 ~ 0.5

highp float shiftedX;
highp float shiftedY;
highp float shiftedX1;
highp float shiftedY1;
highp float shiftedX2;
highp float shiftedY2;

highp float sliceCount;

highp float fixShiftedValue(highp float shfitedValue) {
    // Keep
    if (shfitedValue > 1.0) {
        return 1.0;
    }
    else if (shfitedValue < 0.0) {
        return 0.0;
    }
    return shfitedValue;
}

highp float random (highp vec2 st) {
    //return fract(sin(dot(st.xy, vec2(12.9898,78.233)))* 43758.5453123);
    // change 43758.5453123 to 43.7585453123 to avoid unexpected noise.
    return fract(sin(dot(st.xy, vec2(12.9898, 78.233)))* 43.7585453123);
}

highp float textureOffset(highp float x){
    x = x + phaseShift;
    return (0.5*sin(45.0*amplitude*x)+0.1*cos(30.0*amplitude*x)+0.25*cos(15.0*amplitude*x-2.0)+0.25*cos(7.5*amplitude*x-1.0))+vDisplacement;
}

highp float texture2Offset(highp float x){
    x = x + phaseShift;
    return (0.5*sin(45.0*amplitude*x+3.0)+0.1*cos(30.0*amplitude*x+2.0)+0.25*cos(15.0*amplitude*x-2.0)+0.25*cos(7.5*amplitude*x-1.0))-vDisplacement;
}

highp float messUpTextureOffset(highp float textureOffset, highp float upperBoundary, highp float lowerBoundary){
    highp float newTextureOffset = textureOffset;
    lowp float randomNum;
    if (direction == 1) {
        randomNum = random(vec2(0.0, textureCoordinate.y));
    } else {
        randomNum = random(vec2(textureCoordinate.x, 0.0));
    }

    if (textureOffset > upperBoundary) {
        if (randomNum > (1.0-noiseScale)){
            newTextureOffset = upperBoundary;
        }
    }
    else if (textureOffset > lowerBoundary) {
        if (randomNum > (1.0-noiseScale)){
            newTextureOffset = upperBoundary+0.1;
        }
        else if (randomNum < noiseScale){
            newTextureOffset = lowerBoundary;
        }
    }
    else {
        if (randomNum > (1.0-noiseScale)){
            newTextureOffset = lowerBoundary;
        }
    }
    return newTextureOffset;
}

void main()
{
    sliceCount = 32.0;

    shiftedX = textureCoordinate.x;
    shiftedY = textureCoordinate.y;
    shiftedX1 = textureCoordinate.x;
    shiftedY1 = textureCoordinate.y;
    shiftedX2 = textureCoordinate.x;
    shiftedY2 = textureCoordinate.y;

    lowp float upperBoundary = 0.4;
    lowp float lowerBoundary = -0.4;

    if (direction == 1) {
        highp float textureOffset = textureOffset(textureCoordinate.y);
        textureOffset = messUpTextureOffset(textureOffset, upperBoundary, lowerBoundary);
        if (textureOffset > upperBoundary) {
            shiftedX = textureCoordinate.x + offset + 0.01;
        }
        else if (textureOffset > lowerBoundary) {
            shiftedX = textureCoordinate.x;
        }
        else {
            shiftedX = textureCoordinate.x- offset - 0.01;
        }
        shiftedX = fixShiftedValue(shiftedX);

        highp float texture2Offset = texture2Offset(textureCoordinate.y);
        texture2Offset = messUpTextureOffset(texture2Offset, upperBoundary, lowerBoundary);
        if (texture2Offset > upperBoundary) {
            shiftedX2 = textureCoordinate.x + offset;
        }
        else if (texture2Offset > lowerBoundary) {
            shiftedX2 = textureCoordinate.x;
        }
        else {
            shiftedX2 = textureCoordinate.x - offset;
        }
        shiftedX2 = fixShiftedValue(shiftedX2);

        if (mod(floor(textureCoordinate.y * sliceCount), 2.0) == 0.0) {
            shiftedX1 = textureCoordinate.x+offset*0.5;
            shiftedX1 = fixShiftedValue(shiftedX1);
        }
        else {
            shiftedX1 = textureCoordinate.x-offset*0.5;
            shiftedX1 = fixShiftedValue(shiftedX1);
        }
    }
    else if (direction == 2) {
        highp float textureOffset = textureOffset(textureCoordinate.x);
        textureOffset = messUpTextureOffset(textureOffset, upperBoundary, lowerBoundary);
        if (textureOffset > upperBoundary) {
            shiftedY = textureCoordinate.y + offset + 0.01;
        }
        else if (textureOffset > lowerBoundary) {
            shiftedY = textureCoordinate.y;
        }
        else {
            shiftedY = textureCoordinate.y- offset - 0.01;
        }
        shiftedY = fixShiftedValue(shiftedY);

        highp float texture2Offset = texture2Offset(textureCoordinate.x);
        texture2Offset = messUpTextureOffset(texture2Offset, upperBoundary, lowerBoundary);
        if (texture2Offset > upperBoundary) {
            shiftedY2 = textureCoordinate.y + offset;
        }
        else if (texture2Offset > lowerBoundary) {
            shiftedY2 = textureCoordinate.y;
        }
        else {
            shiftedY2 = textureCoordinate.y - offset;
        }
        shiftedY2 = fixShiftedValue(shiftedY2);

        if (mod(floor(textureCoordinate.x * sliceCount), 2.0) == 0.0) {
            shiftedY1 = textureCoordinate.y+offset*0.5;
            shiftedY1 = fixShiftedValue(shiftedY1);
        }
        else {
            shiftedY1 = textureCoordinate.y-offset*0.5;
            shiftedY1 = fixShiftedValue(shiftedY1);
        }
    }

    lowp vec4 textureColor = texture2D(inputImageTexture, vec2(shiftedX, shiftedY));
    lowp vec4 textureColor2 = texture2D(inputImageTexture, vec2(shiftedX2, shiftedY2));

    if (1 <= colorMode && colorMode <= 3) {
        lowp vec4 textureColor1 = texture2D(inputImageTexture, vec2(shiftedX1, shiftedY1));
        if (colorMode == 1) {
            gl_FragColor = vec4(textureColor.r, textureColor1.g, textureColor2.b, 1.0);
        }
        else if (colorMode == 2) {
            gl_FragColor = vec4(textureColor.r, textureColor2.g, textureColor1.b, 1.0);
        }
        else if (colorMode == 3) {
            gl_FragColor = vec4(textureColor1.r, textureColor.g, textureColor2.b, 1.0);
        }
    }
    else {
        if (colorMode == 4) {
            gl_FragColor = vec4(textureColor2.r, textureColor.g, textureColor2.b, 1.0);
        }
        else if (colorMode == 5) {
            gl_FragColor = vec4(textureColor.r, textureColor.g, textureColor2.b, 1.0);
        }
        else if (colorMode == 6) {
            gl_FragColor = vec4(textureColor2.r, textureColor.g, textureColor.b, 1.0);
        }
        else {
            gl_FragColor = textureColor;
        }
    }
}
