varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform int displayType;
uniform lowp float density;
uniform lowp float wHRatio;

mediump float baseSliceCount = 3.0;
mediump float maxSliceCount = 300.0;
uniform highp vec2 circleCenter;

highp vec2 adjustPoint(highp vec2 oldPoint)
{
    highp vec2 newPoint = vec2(oldPoint);
    newPoint.x = oldPoint.x * wHRatio;
    //newPoint.y = oldPoint.y / wHRatio;
    return newPoint;
}

void main()
{
    lowp vec4 oriTextureColor = texture2D(inputImageTexture, textureCoordinate);
    lowp vec4 newTextureColor = vec4(0.0, 0.0, 0.0, 1.0);

    mediump float sliceCount = baseSliceCount + (maxSliceCount - baseSliceCount) * density;
    mediump float modValue = 0.0;
    lowp float mixValue = 0.0;

    if (displayType == 1) {
        // Horizontal
        modValue = mod(floor(textureCoordinate.y * sliceCount), baseSliceCount);
    }
    else if (displayType == 2) {
        // Vertical
        modValue = mod(floor(textureCoordinate.x * sliceCount), baseSliceCount);
    }
    else {
        // Circle
        modValue = mod(floor(distance(adjustPoint(textureCoordinate), adjustPoint(circleCenter)) * sliceCount * 2.0), baseSliceCount);
    }

    if (modValue == 0.0) {
        mixValue = 0.15;
    }
    else if (modValue == 1.0) {
        mixValue = 0.1;
    }

    gl_FragColor = mix(oriTextureColor, newTextureColor, mixValue);
}
