varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform highp float radius;
uniform highp float aspectRatio;
uniform highp float angle;
uniform highp vec2 centerPosition;
uniform highp int filterType;

void main()
{
    highp vec2 currentPositionToUse = vec2(0.0);
    highp vec2 centerPositionToUse = vec2(0.0);
    highp float r = 0.0;
    highp float transformRate = 0.0;
    if (aspectRatio > 1.0) {
        currentPositionToUse = vec2(textureCoordinate.x, textureCoordinate.y * aspectRatio + 0.5 - 0.5 * aspectRatio);
    } else {
        currentPositionToUse = vec2(textureCoordinate.x / aspectRatio + 0.5 - 0.5 / aspectRatio, textureCoordinate.y );
    }
    if (aspectRatio > 1.0) {
        centerPositionToUse = vec2(centerPosition.x, centerPosition.y * aspectRatio + 0.5 - 0.5 * aspectRatio);
    } else {
        centerPositionToUse = vec2(centerPosition.x / aspectRatio + 0.5 - 0.5 / aspectRatio, centerPosition.y );
    }

    if (filterType == 1) {
        r = distance(centerPositionToUse, currentPositionToUse);
    } else {
        r = (currentPositionToUse.x - centerPositionToUse.x)*cos(angle) + (currentPositionToUse.y- centerPositionToUse.y)*sin(angle);
    if (r < 0.0) {
        r = r * -1.0;
        }
    }

    lowp vec4 firstColor = texture2D(inputImageTexture,textureCoordinate);
    lowp vec4 secondColor = texture2D(inputImageTexture2,textureCoordinate2);
    if (r > radius) {
        gl_FragColor = firstColor;
    } else if ( r > (radius * 0.5) ) {
        transformRate = (r - (radius * 0.5))/(radius * 0.5);
        gl_FragColor = transformRate*firstColor+(1.0 - transformRate)*secondColor;
    } else {
        gl_FragColor = secondColor;
    }
}