varying lowp vec2 textureCoordinate;
varying lowp vec2 maskCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTextureMask;
uniform highp float bokehSize;
uniform highp vec2 imageStep;
uniform highp vec2 maskStep;
uniform highp int windowSize;
uniform highp int iterations;
uniform highp float highlightMultiply;
uniform highp float highlightThreshold;

highp vec4 getHighlight(highp vec2 coord) {
   highp vec4 cr = vec4(texture2D(inputImageTexture, coord).rgb, 1.0);
   highp float m = max(max(cr.r, cr.g), cr.b);
   if (m > highlightThreshold) {
       cr *= highlightMultiply;
   }
   return cr;
}

void main()
{
   highp vec2  multiplier   = vec2(0.0);
   highp vec4  res          = vec4(0.0);
   if(bokehSize == 0.0)
   {
       gl_FragColor = texture2D(inputImageTexture,textureCoordinate);
       return;
   }
   for(int i = 0; i < iterations; i++)
   {
       for(int j = 0; j < iterations; j++)
       {
           highp vec2 coord    = vec2(j,i);
           highp vec2 mask_pos = vec2(1.0)-coord*maskStep;
           highp float mask_weight = texture2D(inputImageTextureMask,mask_pos).r;
           if(mask_weight < 0.0001)
           {
               continue;
           }

           multiplier = coord-vec2((float(iterations)-1.0)/2.0);
           highp vec2 img_pos = textureCoordinate+multiplier*imageStep;
           res += getHighlight(img_pos)*mask_weight;
       }
   }
   gl_FragColor = vec4(res.rgb/res.w,1.0);
}